# Microeconometrics
# CLASS 1
# Continous variables
# Wiktor Budzinski, Marek Giergiczny

rm(list=ls())

# install.packages("readxl") # Install packages if necessary 

library(readxl)
library(rcompanion)
library(jtools)
library(lmtest)
library(tseries)
library(MASS) 
library(splines)
library(glmnet)
# library(lmvar)
library(car)
library(mgcv)
library(statmod)
library(stargazer)
library(mltools)

################################################# Exercise 1 ########################################
# ctrl+l
setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 1")        # sets the access path (sometimes change the characters '\' to '/' is needed)
data <- read_excel("wine.xlsx")
View(data) 

hist(data$Price) # histogram of data
summary(data$Price) # Basic information about variables

summary(data$Cases) 
data$Cases <- data$Cases/100 # Scalling the covariate
plot(data$Cases, data$Price) # simple plot

# Different ways to define equations in R: 

Eq1a <- Price ~ Cases + Score + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99

model1 <- lm(Eq1a, data=data)
summary(model1)
summ(model1, digits = 4) # different function to present the results

Eq1b <- Price ~ .
model1b = lm(Eq1b, data=data)
summ(model1b, digits = 4)


Eq1c<-reformulate(names(data)[-1], names(data)[1])
model1c <- lm(Eq1c, data=data)
summ(model1c, digits = 4)

# Easy way to compare models next to each other
stargazer(model1, model1b,model1c, type="text", df=FALSE)  

resettest(model1)

# Test RESET conducted 'by hand'
data$mu1 <- model1$fitted.values # fitted values
Eq1d <- Price ~ Cases + Score + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99 + 
  I(mu1^2)+ I(mu1^3)
model1d <- lm(Eq1d, data=data)
summ(model1d, digits = 4)
linearHypothesis(model=model1d, c("I(mu1^2)=0","I(mu1^3)=0")) # Test RESET

# Breush-Pagan test
bptest(model1)

# Breush-Pagan test 'by hand'
data$err2 <- rstandard(model1)^2
Eq1e <- err2 ~ Cases + Score + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99
model1e <- lm(Eq1e, data=data)
summ(model1e, digits = 4)
summary(model1e)$r.squared*9600

jarque.bera.test(model1$residuals) # test for normality

# Useful graphical tests
plotNormalHistogram(model1$residuals) # skewed

scatter.smooth(rstandard(model1) ~ data$Cases, col="grey",
               las=1, ylab="Standardized residuals", xlab="No. of produced cases")
scatter.smooth(rstandard(model1) ~ data$Score, col="grey",
               las=1, ylab="Standardized residuals", xlab="Taste Score")
scatter.smooth(rstandard(model1) ~ model1$fitted.values, col="grey",
               las=1, ylab="Standardized residuals", xlab="Fitted values")

################################################# Exercise 2 ########################################

plot(data$Cases, data$Price)
plot(data$Score, data$Price)

data$LogPrice <- log(data$Price)
Eq2 <- LogPrice ~ Cases + Score + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99 
model2 <- lm(Eq2, data=data)
summ(model2, digits = 4)

# Some graphical checks
plotNormalHistogram(model2$residuals)
scatter.smooth(rstandard(model2) ~ data$Cases, col="grey",
               las=1, ylab="Standardized residuals", xlab="No. of produced cases")
resettest(model2)

data$LogCases <- log(data$Cases)
data$LogScore <- log(data$Score)
Eq3 <- LogPrice ~ LogCases + LogScore + factor(Age) + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99 
model3 <- lm(Eq3, data=data)
summ(model3, digits = 4)

scatter.smooth(rstandard(model3) ~ data$Cases, col="grey",
               las=1, ylab="Standardized residuals", xlab="No. of produced cases")
scatter.smooth(rstandard(model3) ~ data$Score, col="grey",
               las=1, ylab="Standardized residuals", xlab="Taste Score")

# Plotting coefficients
plot_summs(model3)
plot_summs(model3, coefs = c("2 years" = "factor(Age)2", "3 years" = "factor(Age)3", "4 years" = "factor(Age)4", 
                             "5 years" = "factor(Age)5", "6 years" = "factor(Age)6"))

linearHypothesis(model=model3, c("factor(Age)2 = factor(Age)3 - factor(Age)2","factor(Age)2 = factor(Age)4 - factor(Age)3",
                                  "factor(Age)2 = factor(Age)5 - factor(Age)4", "factor(Age)2 = factor(Age)6 - factor(Age)5")) 

data$ScoreSq <- (data$Score/10)^2
Eq4 <- LogPrice ~ LogCases + Score + ScoreSq + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99 
model4 <- lm(Eq4, data=data)
summ(model4, digits = 4)
scatter.smooth(rstandard(model4) ~ data$Score, col="grey",
               las=1, ylab="Standardized residuals", xlab="Taste Score")
resettest(model4)

# You can try to recode continous variable as a categorical one

data$CasesBin <- bin_data(data$Cases, bins=10, binType="quantile") # equal no. of observations in each category
#data$CasesBin2 <- bin_data(data$Cases, bins=10) # equaly spaced categories
data$ScoreBin <- bin_data(data$Score, bins=10, binType="quantile") # equal no. of observations in each category
#data$ScoreBin2 <- bin_data(data$Score, bins=10) # equaly spaced categories

Eq5 <- LogPrice ~ factor(data$CasesBin) + factor(data$ScoreBin) + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99 
model5 <- lm(Eq5, data=data)
summ(model5, digits = 4)

data$CasesBin <-unclass(data$CasesBin)
data$ScoreBin <-unclass(data$ScoreBin)
model5 <- lm(Eq5, data=data)
summ(model5, digits = 4)


plot_summs(model5, coefs = c("factor(data$CasesBin)2", "factor(data$CasesBin)3", "factor(data$CasesBin)4", 
                             "factor(data$CasesBin)5", "factor(data$CasesBin)6", "factor(data$CasesBin)7",
                             "factor(data$CasesBin)8", "factor(data$CasesBin)9", "factor(data$CasesBin)10"))

plot_summs(model5, coefs = c("factor(data$ScoreBin)2", "factor(data$ScoreBin)3", "factor(data$ScoreBin)4", 
                             "factor(data$ScoreBin)5", "factor(data$ScoreBin)6", "factor(data$ScoreBin)7",
                             "factor(data$ScoreBin)8", "factor(data$ScoreBin)9"))


Eq6 <- LogPrice ~ LogCases + Score + ScoreSq + Age + NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99 
model6 <- lm(Eq6, data=data)
summ(model6, digits = 4)

# Add interaction

Eq7 <- LogPrice ~ LogCases + LogCases:washington + LogCases:Nonvarietal + LogCases:Pinotnoir + LogCases:Cabernet + LogCases:Merlot + LogCases:Shyrah +
  Score + ScoreSq + Score:washington + Score:Nonvarietal + Score:Pinotnoir + Score:Cabernet + Score:Merlot + Score:Shyrah +
  Age + Age:washington + Age:Nonvarietal + Age:Pinotnoir + Age:Cabernet + Age:Merlot + Age:Shyrah +
  NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99
model7 <- lm(Eq7, data=data)
summ(model7, digits = 4)
resettest(model7)

################################################# Exercise 3 ########################################
# Influential observations

data$h <- hatvalues(model7) # leverage
sort(data$h, decreasing=TRUE)[1:10]
mean(data$h) # large values of h (perhaps two or three times the mean value of the h) identify unusual combinations of the explanatory variables
hist(data$h)

plot(data$h, abs(model7$residuals)) # High leverage vs. outliers

# Influential measures 
cd<- cooks.distance(model7) # is it greater than 1?
hist(cd)
dff<-dffits(model7) # Similar to cooks distance
db<-dfbetas(model7)

# Influential measures with a single function
im <- influence.measures(model7)
head(im$infmat)
colSums(im$is.inf) # hat is leverage
head(im$is.inf)
median(data$Price[which(im$is.inf[,49])]) # DFFITS
median(data$Price)
mean(data$Price[which(im$is.inf[,49])]> mean(data$Price)) # what % is higher than the average price
scatter.smooth(abs(im$infmat[,49]) ~ data$Price, col="grey",
               las=1, ylab="DFFit", xlab="Price")

save(data, file = "Class1.RData")